#include "NGTP-IF1-ControlServices.h"
#include "rtxsrc/rtxCommon.h"
#include "rtxmlsrc/rtXmlCppMsgBuf.h"

#include <stdio.h>
#include <stdlib.h>

int main (int argc, char** argv)
{
   const OSOCTET* msgptr;
   OSBOOL     aligned = TRUE;
   OSBOOL     trace = TRUE;
   OSBOOL     verbose = FALSE;
   FILE*      fp;
   char*      filename = "message.dat";
   int        i, len, stat;

   /* Process Command line arguments */
   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-a")) aligned = TRUE;
         else if (!strcmp (argv[i], "-u")) aligned = FALSE;
         else if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-o")) filename = argv[++i];
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else {
            printf ("usage: writer -a | -u [ -v ] [ -o <filename>\n");
            printf ("   -a  use PER aligned encoding\n");
            printf ("   -u  use PER unaligned encoding\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -o <filename>  write encoded msg to <filename>\n");
            printf ("   -notrace  do not display trace info\n");
            return 1;
         }
      }
   }

   // Create an instance of the compiler generated class.
   // This example uses a dynamic message buffer..
   ASN1PEREncodeBuffer encodeBuffer (aligned);
   ASN1T_ControlData data;
   ASN1C_ControlData ControlDataPDU (encodeBuffer, data);

   // Populate structure of generated type
   data.controlHeader.nodeId = "myNodeId";
   data.controlHeader.applicationContext = 9765;
   data.controlHeader.sessionCounter = 9617;
   data.controlHeader.initiator = ControlHeader_initiator::tu;

   ASN1T_ApplicationToApplication applToAppl;
   ASN1C_ServicesBlock_descriptors 
      srvBlkDescr (ControlDataPDU, applToAppl.servicesBlock.descriptors);

   ASN1T_ControlServiceDescriptor* pcsd;
   pcsd = srvBlkDescr.NewElement();
   OSRTASSERT (0 != pcsd);
   pcsd->category = 22;
   pcsd->implementation = 33;
   srvBlkDescr.Append (pcsd);

   pcsd = srvBlkDescr.NewElement();
   OSRTASSERT (0 != pcsd);
   pcsd->category = 44;
   pcsd->implementation = 55;
   srvBlkDescr.Append (pcsd);

   const OSOCTET serviceData[] = {
      0x12, 0x34, 0x56, 0x78, 0x9A
   } ;
   applToAppl.servicesBlock.serviceData.numocts = sizeof(serviceData);
   applToAppl.servicesBlock.serviceData.data = serviceData;

   data.controlBody.t = T_ControlBody_applicationToApplication;
   data.controlBody.u.applicationToApplication = &applToAppl;

   // Encode 
   encodeBuffer.setTrace (trace);

   if ((stat = ControlDataPDU.Encode ()) == 0)
   {
      if (trace) {
         printf ("Encoding was successful\n");
         printf ("Hex dump of encoded record:\n");
         encodeBuffer.hexDump ();
         printf ("Binary dump:\n");
         encodeBuffer.binDump ("Data");
      }
      msgptr = encodeBuffer.getMsgPtr ();
      len = encodeBuffer.getMsgLen ();
   }
   else
   {
      printf ("Encoding failed\n");
      encodeBuffer.printErrorInfo ();
      return (-1);
   }

   // Write the encoded message out to the output file
   fp = fopen (filename, "wb");
   if (0 != fp) {
      fwrite (msgptr, 1, len, fp);
      fclose (fp);
   }
   else {
      printf ("Error opening %s for write access\n", filename);
      return -1;
   }

   return 0;
}
